package cs451.net.event;

import cs451.net.event.message.Message;
import cs451.net.event.message.NetMessageAbstract;
import cs451.parser.Host;

/**
 * NetEvent abstraction class
 *
 * Holds a {@link Message} and a {@link cs451.parser.Host}
 *
 * @author C. Hölzl
 */
public class NetEvent {

    public final Host peer;
    public final NetMessageAbstract message;

    private NetEvent(Host peer, NetMessageAbstract m) {
        this.peer = peer;
        this.message = m;
    }

    public static NetEvent Message(NetMessageAbstract m){
        return new NetEvent(NO_PEER, NetMessageAbstract.MSG(m));
    }

    public static NetEvent Message(Host peer, NetMessageAbstract m){
        return new NetEvent(peer, NetMessageAbstract.MSG(m));
    }

    public static NetEvent MessageACK(Host peer, NetMessageAbstract m) {
        return new NetEvent(peer,NetMessageAbstract.ACK(m));
    }

    public static NetEvent MessageHRTB(Host peer){
        return new NetEvent(peer,NetMessageAbstract.HRTB());
    }

    public static NetEvent MessageCRSH(Host peer){
        return new NetEvent(peer,NetMessageAbstract.EMPTY());
    }

    public static NetEvent EMPTY(){
        return new NetEvent(NO_PEER, NetMessageAbstract.EMPTY());
    }

    public static final Host NO_PEER = null;
}