package cs451.net.event.message;

import java.nio.ByteBuffer;
import java.util.HashMap;


/**
 * Message abstraction class
 *
 * @author C. Hölzl
 */
public class Message extends NetMessageAbstract {

    public final HashMap<Integer,Integer> dependencies;

    private Message(Integer src, Integer id, TYPE tpe, HashMap<Integer,Integer> dependencies){
        super(src, id, tpe);
        this.dependencies = dependencies;
    }

    /**
     * Create a message from a ByteBuffer
     * @param b buffer
     * @return Message
     */
    public static Message FromBuffer(ByteBuffer b) {
        if(b.remaining() < 14) return getBuilder().build();

        Character tpe = b.getChar();
        int id = b.getInt();
        int src = b.getInt();
        int depLen = b.getInt();
        HashMap<Integer,Integer> newdep = new HashMap<>(depLen);
        for (int i = 0; i < depLen; i++) {
            if(b.remaining()<8) break;
            newdep.put(b.getInt(),b.getInt());
        }
        return new Message(src, id, NetMessageInterface.CharacterToTpe(tpe), newdep);
    }

    /**
     * Puts a message into a ByteBuffer
     * @param b buffer
     */
    public void ToBuffer(ByteBuffer b){
        if(b.remaining() < 14 + 8*dependencies.keySet().size()) return;

        b.putChar(tpe.c);
        b.putInt(id);
        b.putInt(src);
        b.putInt(dependencies.keySet().size());
        for (Integer i : dependencies.keySet()) {
            b.putInt(i);
            b.putInt(dependencies.get(i));
        }
    }

    @Override
    public String toString() {
        return src + "-" + tpe.c + id + " - " +dependencies.toString();
    }

    public static final Message HRTB_MSG = Message.getBuilder().setId(0).setTpe(TYPE.HRTB).build();

    public static MessageBuilder getBuilder(){
        return new MessageBuilder();
    }
    public static MessageBuilder getBuilder(NetMessageAbstract m){
        if (m instanceof Message)
            return new MessageBuilder((Message) m);
        return getBuilder();
    }

    public static class MessageBuilder{
        private int id = -1;
        private int src = -1;
        private TYPE tpe = TYPE.NONE;
        private HashMap<Integer,Integer> dependencies = new HashMap<>(0);

        public MessageBuilder(){}
        public MessageBuilder(Message m){
            this.id = m.id;
            this.src = m.src;
            this.tpe = m.tpe;
            this.dependencies = m.dependencies;
        }

        public Message build(){
            return new Message(src,id,tpe,dependencies);
        }

        public MessageBuilder setTpe(TYPE tpe) {
            this.tpe = tpe;
            return this;
        }

        public MessageBuilder setId(int id) {
            this.id = id;
            return this;
        }

        public MessageBuilder setSrc(int src) {
            this.src = src;
            return this;
        }

        public MessageBuilder setDependencies(HashMap<Integer,Integer> dependencies) {
            this.dependencies = dependencies;
            return this;
        }
    }
}
