package cs451.net.event.message;

import cs451.net.NetManager;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

public class MessageBundle extends NetMessageAbstract {

    public final List<Message> messageList;

    private MessageBundle(int src, int id, TYPE tpe, List<Message> messageList) {
        super(src,id,tpe);
        this.messageList = messageList;
    }

    public static NetMessageAbstract FromBuffer(ByteBuffer b) {
        ArrayList<Message> msgl = new ArrayList<>();
        int mlen = Math.min(NetManager.BUNDLE_SIZE+1, b.getInt());
        for (int i = 0; i< mlen; i++) {
            Message m = Message.FromBuffer(b);
            if (m.tpe == TYPE.NONE)
                break;
            msgl.add(m);
        }
        return getBuilder().setMessageList(msgl).build();
    }
    @Override
    public void ToBuffer(ByteBuffer b) {
        if (b.remaining() < 4) return;

        b.putInt(messageList.size());
        for (Message message : messageList) {
            message.ToBuffer(b);
        }
    }

    @Override
    public String toString() {
        return tpe.c + "-" + Arrays.toString(messageList.toArray());
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final MessageBundle m = (MessageBundle) obj;
        return messageList.equals(m.messageList);
    }

    @Override
    public int hashCode() {
        return Objects.hash(messageList);
    }

    @Override
    public int compareTo(NetMessageAbstract message) {
        return 0;
    }

    public static MessageBundleBuilder getBuilder(){
        return new MessageBundleBuilder();
    }
    public static MessageBundleBuilder getBuilder(NetMessageAbstract m){
        if (m instanceof  MessageBundle)
            return new MessageBundleBuilder((MessageBundle) m);
        return getBuilder();
    }

    public static class MessageBundleBuilder{
        private int id = -1;
        private int src = -1;
        private TYPE tpe = TYPE.NONE;
        private List<Message> messageList = new ArrayList<>(0);

        public MessageBundleBuilder(){}
        public MessageBundleBuilder(MessageBundle m){
            this.id = m.id;
            this.src = m.src;
            this.tpe = m.tpe;
            this.messageList = m.messageList;
        }

        public MessageBundle build(){
            return new MessageBundle(src,id,tpe,messageList);
        }

        public MessageBundleBuilder setTpe(TYPE tpe) {
            this.tpe = tpe;
            return this;
        }

        public MessageBundleBuilder setId(int id) {
            this.id = id;
            return this;
        }

        public MessageBundleBuilder setSrc(int src) {
            this.src = src;
            return this;
        }

        public MessageBundleBuilder setMessageList(List<Message> dependencies) {
            this.messageList = dependencies;
            return this;
        }
    }
}
