package cs451.net.event.message;

import java.util.HashMap;
import java.util.List;
import java.util.Objects;

public abstract class NetMessageAbstract implements NetMessageInterface {
    public final TYPE tpe;
    public final Integer src;
    public final Integer id;

    public NetMessageAbstract(Integer src,Integer id, TYPE tpe) {
        this.src = src;
        this.id = id;
        this.tpe = tpe;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final NetMessageAbstract m = (NetMessageAbstract) obj;
        return id.equals(m.id) && src.equals(m.src);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id,src);
    }

    @Override
    public int compareTo(NetMessageAbstract m) {
        return id.compareTo(m.id);
    }


    public static NetMessageAbstract MSG(NetMessageAbstract m) {
        if (m instanceof  Message)
            return Message.getBuilder(m).build();
        if (m instanceof  MessageBundle)
            return MessageBundle.getBuilder(m).build();

        return EMPTY();
    }

    /**
     * Build a Ack Message
     * @return Message
     */
    public static NetMessageAbstract ACK(NetMessageAbstract m){
        if(m instanceof Message)
            return Message.getBuilder(m).setTpe(TYPE.ACK).setDependencies(new HashMap<>()).build();
        return EMPTY();
    }


    /**
     * Build a Heartbeat Message
     * @return Message
     */
    public static NetMessageAbstract HRTB(){
        return MessageBundle.getBuilder().setMessageList(List.of(Message.HRTB_MSG)).setTpe(TYPE.HRTB).build();
    }

    public boolean isHeartBeat(){
        if (this instanceof Message){
            return this.tpe==TYPE.HRTB;
        }else if (this instanceof MessageBundle){
            return ((MessageBundle)this).messageList.size()>0 &&
                    ((MessageBundle)this).messageList.get(0).tpe==TYPE.HRTB;
        }else{
            return false;
        }
    }

    /**
     * @return Empty Message with no type nor ID
     */
    public static Message EMPTY(){
        return Message.getBuilder().build();
    }
}
