package cs451.net.event.message;

import java.nio.ByteBuffer;
import java.util.Arrays;

public interface NetMessageInterface extends Comparable<NetMessageAbstract> {

    /**
     * Type of Message
     */
    enum TYPE {
        NONE("NONE", 'N'),
        DATA("DATA", 'D'),
        ACK("ACK", 'A'),
        HRTB("HRTB",'H');

        public final Character c;
        public final String tag;
        TYPE(String tag, Character c){
            this.tag = tag;
            this.c = c;
        }
    }

    /**
     * Converts a character to a given type
     * @param c char to convert
     * @return Type
     */
    static TYPE CharacterToTpe(Character c){
        return Arrays.stream(TYPE.values()).filter(type -> type.c==c).findFirst().orElse(TYPE.NONE);
    }

    /**
     * Create a message from a ByteBuffer
     *
     * @param b buffer
     * @return Message
     */
    static NetMessageAbstract FromBuffer(ByteBuffer b) {
        return MessageBundle.FromBuffer(b);
    }

    /**
     * Puts a message into a ByteBuffer
     * @param b buffer
     */
     void ToBuffer(ByteBuffer b);
}
