package cs451.net.handler;

import cs451.net.NetManager;
import cs451.net.event.NetEvent;
import cs451.net.event.message.Message;
import cs451.net.event.message.MessageBundle;
import cs451.parser.Host;
import cs451.parser.Parser;

import java.util.ArrayList;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.PriorityBlockingQueue;

/**
 * NetEventHandler for BLUNDLER
 *
 * Groups Multiple messages into larger packets (and de-groups them on receive
 *
 * @author C. Hölzl
 */
public class NetHandlerBNDL extends NetEventHandlerAbstract {

    private static final ConcurrentHashMap<Integer, PriorityBlockingQueue<Message>> sending = new ConcurrentHashMap<>();

    public NetHandlerBNDL(Class<? extends NetEventHandlerAbstract> deliverLayer, Class<? extends NetEventHandlerAbstract> broadcastLayer) {
        super(deliverLayer, broadcastLayer);
    }

    @Override
    public synchronized void broadcastIf() {
        NetManager.hosts.forEach(h-> sending.computeIfPresent(h.getId(),(hid, mq)->{
            while(mq.size()>0){
                ArrayList<Message> msgl = new ArrayList<>(NetManager.BUNDLE_SIZE);
                mq.drainTo(msgl, NetManager.BUNDLE_SIZE);
                broadcastNextAsync(NetEvent.Message(h, MessageBundle.getBuilder().setMessageList(msgl).build()));
            }
            return mq;
        }));
    }

    @Override
    public void broadcast(NetEvent ne) {
        sending.computeIfPresent(ne.peer.getId(),(h, q) -> {
            if(ne.message instanceof Message) {
                q.add((Message) ne.message);
                requestCondBD(true);
            }
            return q;
        });
    }

    @Override
    public void deliver(NetEvent ne) {
        if (ne.message instanceof MessageBundle) {
            ((MessageBundle) ne.message).messageList.forEach(msg -> deliverNextAsync(NetEvent.Message(ne.peer,msg)));
        }
    }

    @Override
    public boolean isDone() {
        return sending.values().stream().allMatch(v->v.size()==0);
    }

    @Override
    public void start(Host h, Parser p) {
        p.hosts().stream().filter(ch->ch.getId()!=h.getId()).forEach(ch->sending.put(ch.getId(),new PriorityBlockingQueue<>()));
    }
}
