package cs451.net.handler;

import cs451.net.NetManager;
import cs451.net.event.NetEvent;
import cs451.parser.Host;
import cs451.parser.Parser;

import java.util.concurrent.ConcurrentHashMap;

/**
 * NetEventHandler for Failure Detection
 *
 * Beat: Sends ping to hosts
 * Deliver: Marks peer as alive (Drop HRBT packets)
 *
 * @author C. Hölzl
 */
public class NetHandlerFD extends NetEventHandlerAbstract {

    private static final ConcurrentHashMap<Integer,Integer> alive = new ConcurrentHashMap<>();
    private static final ConcurrentHashMap<Integer,Integer> should_I = new ConcurrentHashMap<>();

    public NetHandlerFD(Class<? extends NetEventHandlerAbstract> deliverLayer, Class<? extends NetEventHandlerAbstract> broadcastLayer) {
        super(deliverLayer, broadcastLayer);
    }

    @Override
    public synchronized void beat() {
        NetManager.hosts.forEach(h-> {
            should_I.computeIfPresent(h.getId(),(k, v) -> {
                if(v>=0)
                    broadcastNextSync(NetEvent.MessageHRTB(h));
                return v+1;
            });
            alive.computeIfPresent(h.getId(), (k,v) -> v+1);
            if(alive.getOrDefault(h.getId(),0) > NetManager.FD_MAX_TRIES){
                crashNextSync(NetEvent.MessageCRSH(h));
                alive.remove(h.getId());
                should_I.remove(h.getId());
            }
        });
    }

    @Override
    public void broadcast(NetEvent ne) {
        should_I.computeIfPresent(ne.peer.getId(),(k, v)->-1);
        if (alive.containsKey(ne.peer.getId())){
            broadcastNextSync(ne);
        }
    }

    @Override
    public void deliver(NetEvent ne) {
        alive.computeIfPresent(ne.peer.getId(), (k,v) -> 0);
        if (!ne.message.isHeartBeat()) {
            deliverNextSync(ne);
        }
    }

    @Override
    public void start(Host h, Parser p) {
        NetManager.hosts.stream().filter(ch->ch.getId()!=h.getId()).forEach(ch-> {
            alive.put(ch.getId(),-1);
            should_I.put(ch.getId(),-1);
        });
    }
}
