package cs451.net.handler;

import cs451.net.event.NetEvent;
import cs451.net.event.message.Message;
import cs451.parser.Host;
import cs451.parser.Parser;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.PriorityBlockingQueue;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

public class NetHandlerLCB extends NetEventHandlerAbstract {

    private static Map<Integer, List<Integer>> causality;
    private static final ConcurrentHashMap<Integer, PriorityBlockingQueue<Message>> undelivered = new ConcurrentHashMap<>();
    private static final ConcurrentHashMap<Integer, AtomicInteger> state = new ConcurrentHashMap<>();

    public NetHandlerLCB(Class<? extends NetEventHandlerAbstract> deliverLayer, Class<? extends NetEventHandlerAbstract> broadcastLayer) {
        super(deliverLayer, broadcastLayer);
    }

    @Override
    public synchronized void deliverIf() {
        AtomicBoolean deliveredOne = new AtomicBoolean(false);
        do {
            deliveredOne.set(false);

            undelivered.forEach((hid, mq) -> {
                AtomicInteger crsn = state.get(hid);
                if(crsn == null) return;
                Collection<Message> msgc = new ArrayList<>();
                mq.drainTo(msgc);
                msgc.forEach(msg ->
                        crsn.getAndUpdate(v -> {
                            if (msg.id.equals(v+1) && checkDependencies(msg.src,msg.dependencies)){
                                deliverNextSync(NetEvent.Message(msg));
                                deliveredOne.set(true);
                                return v+1;
                            }else{
                                mq.add(msg);
                                return v;
                            }
                        })
                );
            });
        }while((deliveredOne.get()));
    }

    @Override
    public void broadcast(NetEvent ne) {
        broadcastNextSync(NetEvent.Message(Message.getBuilder(ne.message).setDependencies(getDependencies(ne.message.src)).build()));
    }

    @Override
    public void deliver(NetEvent ne) {
        undelivered.computeIfPresent(ne.message.src,(k, lv) -> {
            lv.add((Message) ne.message);
            return lv;
        });
        requestCondBD(true);
    }

    @Override
    public void start(Host h, Parser p) {
        super.start(h, p);
        causality = p.causality();
        for (Host host : p.hosts()) {
            undelivered.put(host.getId(), new PriorityBlockingQueue<>());
            state.put(host.getId(), new AtomicInteger(0));
        }
    }

    @Override
    public boolean isDone() {
        return undelivered.values().stream().allMatch(AbstractCollection::isEmpty);
    }

    private HashMap<Integer,Integer> getDependencies(Integer hid){
        HashMap<Integer,Integer> dependencies = new HashMap<>();
        List<Integer> hc = causality.getOrDefault(hid, Collections.emptyList());
        for (Integer hci : hc) {
            dependencies.put(hci, state.get(hci).get());
        }
        return dependencies;
    }

    private Boolean checkDependencies(Integer hid, HashMap<Integer,Integer> md){
        HashMap<Integer,Integer> cd = getDependencies(hid);
        for(Integer k : cd.keySet()){
            if(cd.get(k) < md.get(k)){
                return false;
            }
        }
        return true;
    }
}
