package cs451.net.handler;

import cs451.net.NetManager;
import cs451.net.event.NetEvent;
import cs451.net.event.NetEventType;
import cs451.net.event.message.Message;
import cs451.net.event.message.NetMessageInterface;
import cs451.parser.Host;
import cs451.parser.Parser;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * NetEventHandler for Top Level
 *
 * SendIf: based on window, add messages to be sent
 *
 * Send: SendIf
 * Deliver: SendIf
 *
 * @author C. Hölzl
 */
public class NetHandlerTOPL extends NetEventHandlerAbstract {

    private static final AtomicInteger sn = new AtomicInteger(1);

    private static final AtomicInteger toSend = new AtomicInteger(0);
    private static final AtomicInteger delivered = new AtomicInteger(0);
    private static final AtomicInteger waiting = new AtomicInteger(0);


    private Host me;

    public NetHandlerTOPL(Class<? extends NetEventHandlerAbstract> deliverLayer, Class<? extends NetEventHandlerAbstract> broadcastLayer) {
        super(deliverLayer, broadcastLayer);
    }

    @Override
    public synchronized void broadcastIf() {
        while (toSend.get() > 0 && waiting.get() < NetManager.WINDOW_WIDTH) {
            toSend.decrementAndGet();
            waiting.incrementAndGet();

            Message newMess = Message.getBuilder().setSrc(me.getId()).setId(sn.getAndIncrement()).setTpe(NetMessageInterface.TYPE.DATA).build();
            NetManager.complete(NetEventType.BCST, NetEvent.Message(me, newMess));
            broadcastNextSync(NetEvent.Message(newMess));
        }
    }


    @Override
    public void broadcast(NetEvent ne) {
        requestCondBD(true);
    }

    @Override
    public synchronized void deliver(NetEvent ne) {
        NetManager.complete(NetEventType.DLVR,ne);
        if (ne.message.src == me.getId()) {
            delivered.incrementAndGet();
            waiting.decrementAndGet();
            requestCondBD(true);
        }
    }

    @Override
    public boolean isDone() {
        return (toSend.get()==0) && (waiting.get()==0);
    }

    @Override
    public void start(Host h, Parser p) {
        me = h;
        toSend.set(p.messageCount());
    }
}
