package cs451.parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class ConfigParser {

    private String path;

    private int messages;
    private final ConcurrentHashMap<Integer,List<Integer>> causality = new ConcurrentHashMap<>();

    public ConfigParser(List<Host> hosts) {
        for (Host host : hosts) {
            causality.put(host.getId(),new ArrayList<>());
        }
    }


    public boolean populate(String value) {
        File file = new File(value);
        path = file.getPath();

        try (BufferedReader br = new BufferedReader(new FileReader(path))) {
            int lineNum = 0;
            for (String line; (line = br.readLine()) != null; lineNum++) {
                System.out.println(line);
                String[] lparts = line.trim().split(" ");
                if (lineNum == 0){
                    messages = Integer.parseInt(lparts[0]);
                }else if (lineNum>=1 && lineNum<=causality.keySet().size()){
                    Integer hid = Integer.parseInt(lparts[0]);
                    List<Integer> vl = causality.get(hid);
                    if(vl != null){
                        for(int i = 1; i < lparts.length; ++i){
                            vl.add(Integer.parseInt(lparts[i]));
                        }
                    }
                }else{
                    break;
                }
            }
        } catch (IOException e) {
            System.err.println("Problem with the hosts file!");
            return false;
        }
        return true;
    }

    public String getPath() {
        return path;
    }

    public int getMessages() {
        return messages;
    }

    public Map<Integer, List<Integer>> getCausality() {
        return causality;
    }
}
